""" Classes for defining neural network weight optimization problems."""

# Author: Genevieve Hayes (modified by Andrew Rollings)
# License: BSD 3 clause

from .weights import (flatten_weights, unflatten_weights, gradient_descent_original)
