""" Classes for defining optimization problem objects."""

# Author: Genevieve Hayes
# License: BSD 3 clause

from .max_k_color_generator import MaxKColorGenerator
from .knapsack_generator import KnapsackGenerator
from .flip_flop_generator import FlipFlopGenerator
from .queens_generator import QueensGenerator
from .tsp_generator import TSPGenerator
from .continuous_peaks_generator import ContinuousPeaksGenerator
