""" Classes for defining optimization problem objects."""

# Author: Genevieve Hayes
# License: BSD 3 clause

from .arith_decay import ArithDecay
from .geom_decay import GeomDecay
from .exp_decay import ExpDecay
from .custom_schedule import CustomSchedule
