""" Classes for defining neural network weight optimization problems."""

# Author: Genevieve Hayes (modified by Andrew Rollings)
# License: BSD 3 clause

from .identity import (identity)
from .relu import (relu)
from .sigmoid import (sigmoid)
from .softmax import (softmax)
from .tanh import (tanh)
